#!/usr/bin/env python3
#
# This script takes a JSON file with the following format as input:
# [
#      {
#          "test": "TestSite.js",
#          "login": "login",
#          "password": "password",
#          "url": [
#              "http://www.site-to-test.com/",
#              "http://www.site-to-test.de/"
#              ]
#      },
#      {
#          "test": "TestAnotherSite.js",
#          "login": "login",
#          "password": "password",
#          "url": [ "http://www.anothersite-to-test.com/" ]
#      },
#      {
#          "test": "TestYetAnotherSite.js",
#          "login" : "",
#          "password": "",
#          "url" : [ "http://example.com" ]
#      },
#      ...
#  ]
#
# It executes test.js for each test script and URL.
#

"""Usage: test-runner.py [options] [credential_filename]"""


import subprocess
import time
import json
import sys
import os
import shutil

from glob import glob
from optparse import OptionParser
from contextlib import contextmanager
from xml.sax.saxutils import escape


parser = OptionParser(usage=__doc__)
parser.add_option('-f',
                  '--format',
                  dest='format',
                  choices=['xml', 'text'],
                  default='text',
                  help='Format of the test results: xml or text.')
parser.add_option('-o',
                  '--output',
                  dest='output',
                  help='Write XML to this filename.')
parser.add_option('-s',
                  '--show-window',
                  dest='show_window',
                  action='store_true',
                  default=False,
                  help='Display each test running in browser window.')
parser.add_option('-t',
                  '--test',
                  dest='script',
                  default='',
                  help='Script to test. Example: --test GMail')
parser.add_option('-l',
                  '--local',
                  dest='local',
                  default='',
                  help='Search script files (*.user.js and *.test.js) in a local folder instead of grabbing one from LP. Example: --local ~/scripts/')
options, args = parser.parse_args()


# Force XML format if the user chooses an output file.
if options.output:
    options.format = 'xml'


STARTED = time.time()
CREDENTIALS_FILENAME = args[0] if args else 'site-credentials'
TIMEOUT = 150 # seconds
XML_TEMPLATE = """
<testsuite errors="{n_errors}" failures="{n_failures}" name="" tests="{n_tests}" time="{runtime}">
{output}
</testsuite>
"""
FAILURE_TEMPLATE = """
<testcase classname="__main__.SiteTest" name="testSite({name} {url} {login})" time="{runtime}">
<failure type="AssertionError">
{message}
</failure>
</testcase>
"""
PASSING_TEMPLATE = """
<testcase classname="__main__.SiteTest" name="testSite({name} {url} {login})" time="{runtime}" />
"""


try:
    with open(CREDENTIALS_FILENAME, 'r') as fd:
        creds = json.loads(fd.read())
except Exception as e:
    exit(e + '\nCould not read JSON from site-credentials file.\n' + __doc__)


tested = []
failed = []
output = []


def grab_remote_lp_userscript(name, filename):
    with open(filename, 'w') as fd:
        args = ('bzr', 'cat', 'lp:unity-webapps-{}/{}'.format(
            name.lower(), filename))
        print(' '.join(args))
        subprocess.Popen(args, stdout=fd).communicate()


@contextmanager
def get_files(name):
    """Download files when needed, and cleanup when done."""
    files = []
    for ftype in ('test', 'user'):
        filename = '{}.{}.js'.format(name, ftype)
        files.append(filename)
        if not options.local:
            grab_remote_lp_userscript(name, filename)
        else:
            userscript = os.path.join(options.local, filename)
            print('Using local copy: "{}"'.format(userscript))
            if not os.path.exists(userscript):
                raise Exception("Could not find userscript file: " + userscript)
            shutil.copy(userscript, os.getcwd())
    yield
    for filename in files:
        try:
            print('Removing {}...'.format(filename))
            os.unlink(filename)
        except Exception as e:
            print('Failed to clean {}: {}'.format(filename, e))
    print()


def run_test_js(name, url, login=None, password=None):
    """Invoke test.js test case appropriately."""
    args = ['./test.js']
    if options.show_window:
        args.append('--show-window')
    if login:
        args.append('--login')
        args.append(login)
    if password:
        args.append('--password')
        args.append(password)
    if url:
        args.append('--url')
        args.append(url)
    args.append('{}.test.js'.format(name))
    print(' '.join(args))

    begin = time.time()

    try:
        testjs = subprocess.Popen(
            args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    except Exception as e:
        print('Failed to launch test.js: ' + e)
        return

    while True:
        time.sleep(1)
        time_lapse = time.time() - begin
        if testjs.poll() is not None:
            break # Process finished, stop waiting
        elif time_lapse > TIMEOUT:
            testjs.terminate()

    log, err = [ data.decode() for data in testjs.communicate() ]
    url = escape(url)

    runtime = time.time() - begin
    if testjs.returncode != 0:
        failed.append(name)
        message = escape(err + '\n\n' + log)
        output.append(FAILURE_TEMPLATE.format(**locals()))
        print(err)
        print(log)
    else:
        output.append(PASSING_TEMPLATE.format(**locals()))


if options.local is not None and len(options.local) != 0 and not options.script:
    userscripts = glob(os.path.join(options.local, '*.user.js'))
    if len(userscripts) == 1:
        options.script = os.path.basename(userscripts[0]).replace('.user.js', '')
        print('Using locally found {} as the default test name.'.format(
            options.script))
    else:
        exit('Not clear which .user.js file to test, please specify --script.')


# Iterate over credentials listed in site-credentials file.
for cred in creds:
    login = cred.get('login')
    password = cred.get('password')
    name = cred['test'].split('.')[0]

    if options.script and name != options.script.split('.')[0]:
        # If a specific test was chosen, don't run anything else
        continue

    # Call test.js appropriately
    with get_files(name):
        for url in cred.get('url', []):
            tested.append(url)
            run_test_js(name, url, login, password)


# Notify of failures.
if options.format == 'xml':
    xml = XML_TEMPLATE.format(
        n_errors=0,
        n_failures=len(failed),
        n_tests=len(tested),
        runtime=time.time() - STARTED,
        output=''.join(output),
    )
    if options.output:
        with open(options.output, 'w') as fd:
            print(xml, file=fd)
    else:
        print(xml)


print('{} FAILURES'.format(len(failed)))
exit(len(failed))



#!/usr/bin/python3

from urllib.parse import quote
from glob import glob
from json import dumps
from polib import pofile
from sys import argv

result = {}

for filename in glob('po/*.po'):
    po = pofile(filename,
                autodetect_encoding=False,
                encoding='utf-8',
                wrapwidth=-1)
    t = {}
    for entry in po:
        if entry.obsolete or entry.msgstr in ('', entry.msgid):
            continue
        t[entry.msgid] = entry.msgstr
    lang = filename[3:-3] # 'po/lang.po' -> 'lang'
    result[lang] = t

if '--debug' in argv:
    from pprint import pprint
    pprint(result)
else:
    print(quote(dumps(result, sort_keys=True)))



#!/usr/bin/python
from gi.repository import Gio
import os
import time
import sys

AFTER_ICON = "unity://running-apps"

UNITY_LAUNCHER_SETTINGS = "com.canonical.Unity.Launcher"
UNITY_LAUNCHER_FAVORITE_KEY = "favorites"

def modify_webapps_in_launcher(new_desktop_files, current_default_favorites, do_install=True):
    if len(new_desktop_files) != 0:
        try:
            settings = Gio.Settings.new (UNITY_LAUNCHER_SETTINGS)
            default_favorites = current_default_favorites
            if not settings.is_writable(UNITY_LAUNCHER_FAVORITE_KEY):
                print "Unity Webapps migration process (preinstallation): 'favorites' key not writable"
                return
            favorites = settings.get_strv(UNITY_LAUNCHER_FAVORITE_KEY)

            if do_install:
                # only append the desktop icons that are not already present in the favorites list if any
                to_modify = [d for d in new_desktop_files if not d in favorites]
            else:
                to_modify = [d for d in new_desktop_files if d in favorites]

            available_desktop_files = [d for d in favorites if ".desktop" in d]

            if do_install:
                new_index = -1
                
                if len(available_desktop_files):
                    new_index = favorites.index(available_desktop_files[-1]) + 1
                elif AFTER_ICON in favorites:
                    new_index = favorites.index(AFTER_ICON)
                    
                if new_index >= 0:
                    new_index = favorites.index(AFTER_ICON)
                    next = favorites[new_index:]
                    favorites[new_index:] = to_modify
                    favorites.extend(next)
                else:
                    favorites.extend(to_modify)
            else:
                favorites = [f for f in favorites if not f in to_modify]

            settings.set_strv(UNITY_LAUNCHER_FAVORITE_KEY, favorites)

            # force sync to avoid race
            settings.sync()
        except Exception, e:
            print "Unity Webapps migration exception:", str(e)

def install_webapps_in_launcher(new_desktop_files, current_default_favorites):
    print 'Webapp migration script: installing', new_desktop_files, 'in launcher favorites'
    modify_webapps_in_launcher(new_desktop_files, current_default_favorites, True)

def remove_webapps_in_launcher(old_desktop_files, current_default_favorites):
    print 'Webapp migration script: removing', old_desktop_files, 'in launcher favorites'
    modify_webapps_in_launcher(old_desktop_files, current_default_favorites, False)

def get_launcher_favorites():
    settings = Gio.Settings.new(UNITY_LAUNCHER_SETTINGS)
    return settings.get_strv(UNITY_LAUNCHER_FAVORITE_KEY)

def cleanup_local_webapps():
    local_webapp_desktop_files = [
        'GoogleCalendargooglecom.desktop',
        'GoogleDocsdocsgooglecom.desktop',
        'GooglePlusplusgooglecom.desktop',
        'Gmailmailgooglecom.desktop',
        'FacebookMessengerfacebookcom.desktop',
        'YouTubeyoutubecom.desktop',
        'Twittertwittercom.desktop',
        'LiveMailmaillivecom.desktop',
        'YahooMailmailyahoocom.desktop',
    ]

    favorite_webapps_found = []

    default_favorites = set()
    current_favorites = set(get_launcher_favorites())

    for local_webapp_desktop_file in local_webapp_desktop_files:
        current_favorite_webapp_name = "application://{0}".format(
            local_webapp_desktop_file)

        local_webapp_desktop_filename = os.path.join(
            os.path.expanduser("~/.local/share/applications")
            , local_webapp_desktop_file)

        if os.path.exists(local_webapp_desktop_filename):
            print "Removing local webapp desktop file:", local_webapp_desktop_filename
            os.remove(local_webapp_desktop_filename)

            if current_favorite_webapp_name in current_favorites:
                favorite_webapps_found.append(current_favorite_webapp_name)

    remove_webapps_in_launcher(favorite_webapps_found, current_favorites);

    # needed so that unity & its filesystem monitors can track down the changes & update its structures
    time.sleep(2)

    current_favorites = set(get_launcher_favorites())
    install_webapps_in_launcher(favorite_webapps_found, current_favorites);


if __name__ == "__main__":
    source = Gio.SettingsSchemaSource.get_default()
    if source.lookup(UNITY_LAUNCHER_SETTINGS, True) is None: # schema not found
        sys.exit(1)
    cleanup_local_webapps()



#!/usr/bin/python
from gi.repository import Gio
import os
import sys

PREINSTALLED_LAUNCHER_DESKTOP_FILES = ["application://ubuntu-amazon-default.desktop",]
AFTER_ICON = "unity://running-apps"

UNITY_LAUNCHER_SETTINGS = "com.canonical.Unity.Launcher"
UNITY_LAUNCHER_FAVORITE_KEY = "favorites"

def get_default_favorites():
    settings = Gio.Settings.new(UNITY_LAUNCHER_SETTINGS)
    settings.delay()
    settings.reset(UNITY_LAUNCHER_FAVORITE_KEY)
    return settings.get_strv(UNITY_LAUNCHER_FAVORITE_KEY)

def install_default_webapps_in_launcher():
    # Workaround for the script triggering on systems where it shouldn't.
    if os.path.exists("/usr/share/glib-2.0/schemas/95_edubuntu-artwork.gschema.override"):
        return

    new_desktop_files = PREINSTALLED_LAUNCHER_DESKTOP_FILES

    if len(new_desktop_files) != 0:
        try:
            settings = Gio.Settings.new (UNITY_LAUNCHER_SETTINGS)
            default_favorites = get_default_favorites()
            # "If there are no common elements between these sets..."
            if not set(default_favorites).intersection(set(new_desktop_files)):
                print "Skipping Unity Webapps migration for what seems to be a vendor-patched schema"
                return
            if not settings.is_writable(UNITY_LAUNCHER_FAVORITE_KEY):
                print "Unity Webapps migration process (preinstallation): 'favorites' key not writable"
                return
            favorites = settings.get_strv(UNITY_LAUNCHER_FAVORITE_KEY)

            # only append the desktop icons that are not already present in the favorites list if any
            to_add = [d for d in new_desktop_files if not d in favorites]
            available_desktop_files = [d for d in favorites if ".desktop" in d]
            new_index = -1

            if len(available_desktop_files):
                new_index = favorites.index(available_desktop_files[-1]) + 1
            elif AFTER_ICON in favorites:
                new_index = favorites.index(AFTER_ICON)

            if new_index >= 0:
                new_index = favorites.index(AFTER_ICON)
                next = favorites[new_index:]
                favorites[new_index:] = to_add
                favorites.extend(next)
            else:
                favorites.extend(to_add)

            settings.set_strv(UNITY_LAUNCHER_FAVORITE_KEY, favorites)

            # force sync to avoid race
            settings.sync()
        except Exception, e:
            print "Unity Webapps migration exception:", str(e)

if __name__ == "__main__":
    source = Gio.SettingsSchemaSource.get_default()
    if source.lookup(UNITY_LAUNCHER_SETTINGS, True) is None: # schema not found
        sys.exit(1)
    install_default_webapps_in_launcher()
